'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  // is `class extends React.Component`?
  function isReactClass(node) {
    var superClass = node.superClass;
    return t.isMemberExpression(superClass) && t.isIdentifier(superClass.object, { name: 'React' }) && t.isIdentifier(superClass.property, { name: 'Component' });
  }

  var bodyVisitor = {
    ClassMethod: function ClassMethod(path) {
      if (path.node.key.name === 'render') {
        this.renderMethod = path;
      } else {
        this.isPure = false;
        path.stop();
      }
    },
    ClassProperty: function ClassProperty(path) {
      var name = path.node.key.name;

      if (path.node.static && (name === 'propTypes' || name === 'defaultProps')) {
        this.properties.push(path);
      } else {
        this.isPure = false;
      }
    },
    MemberExpression: function MemberExpression(path) {
      var node = path.node;

      // non-this member expressions dont matter

      if (!t.isThisExpression(node.object)) {
        return;
      }

      // Don't allow this.<anything other than props>
      if (!t.isIdentifier(node.property, { name: 'props' })) {
        this.isPure = false;
        path.stop();
        return;
      }

      // this.props.foo => props.foo
      path.replaceWith(t.identifier('props'));
    }
  };

  return {
    visitor: {
      ClassDeclaration: function ClassDeclaration(path) {
        if (!isReactClass(path.node)) {
          // yo, fuck this class then.
          return;
        }

        var state = {
          renderMethod: null,
          properties: [],
          isPure: true
        };

        // get the render method and make sure it doesn't have any other methods
        path.traverse(bodyVisitor, state);

        if (!state.isPure || !state.renderMethod) {
          // fuck this class too.
          return;
        }

        var id = t.identifier(path.node.id.name);

        path.replaceWith(t.functionDeclaration(id, [t.identifier('props')], state.renderMethod.node.body));

        if (!state.properties.length) {
          return;
        }

        state.properties.reverse().forEach(function (prop) {
          path.insertAfter(t.expressionStatement(t.assignmentExpression('=', t.MemberExpression(id, prop.node.key), prop.node.value)));
        });
      }
    }
  };
};